from SciQLopPlots import SciQLopPlot, SciQLopMultiPlotPanel, ProductsModel, Icons, ProductsModelNode, ProductsModelNodeType, ProductsView, ParameterType
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtCore import Qt
import sys
import os
import json

os.environ['QT_API'] = 'PySide6'

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow

__HERE__ = os.path.dirname(__file__)


def filter_metadata(json_node):
    meta = {}
    for k, v in json_node.items():
        if k.startswith("__spz_"):
            continue
        if not isinstance(v, dict):
            meta[k] = v
    return meta

def filter_children(json_node):
    children = {}
    for k, v in json_node.items():
        if isinstance(v, dict):
            children[k] = v
    return children

def load_node(path, json_node):
    if json_node["__spz_type__"] == "ParameterIndex":
        node = ProductsModelNode(json_node["__spz_uid__"],"", filter_metadata(json_node), ProductsModelNodeType.PARAMETER, ParameterType.Multicomponents, "", None)
    else:
        node = ProductsModelNode(json_node["__spz_uid__"], filter_metadata(json_node), "folder_open")

    ProductsModel.instance().add_node(path, node)
    for k, v in filter_children(json_node).items():
        load_node(path + [k], v)

def load_products():
    with open(os.path.join(__HERE__, "products.json")) as f:
        products = json.load(f)
    load_node([], products)


if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(ProductsView(w), "Products")
    load_products()

    w.show()
    app.exec()



from PySide6 import QtCore, QtGui, QtWidgets, QtOpenGL, QtPrintSupport, QtSvg
from . import SciQLopPlotsBindings
from .SciQLopPlotsBindings import (GraphType, SciQLopPlot, SciQLopTimeSeriesPlot, SciQLopMultiPlotPanel,
                                    SciQLopGraphInterface, AxisType, SciQLopPlotRange, SciQLopNDProjectionPlot)
from .SciQLopPlotsBindings import *
from datetime import datetime, timezone
from dateutil.parser import parse as dateparse
import traceback

import sys

sys.modules["SciQLopPlotsBindings"] = SciQLopPlotsBindings

__version__ = '0.17.0'

def _merge_kwargs(kwargs, **kwargs2):
    for k, v in kwargs2.items():
        if k not in kwargs and v is not None:
            kwargs[k] = v
    return kwargs

def _patch_sciqlop_plot(cls):
    def plot_func(self, callback, graph_type=None, **kwargs):
        try:
            kwargs = {k: v for k, v in kwargs.items() if v is not None}
            res = None
            if graph_type == GraphType.ParametricCurve:
                res = cls.parametric_curve(self, callback, **kwargs)
            if graph_type == GraphType.Line:
                res = cls.line(self, callback, **kwargs)
            if graph_type == GraphType.ColorMap:
                res = cls.colormap(self, callback, **kwargs)
            return res
        except Exception as e:
            print(f"Error in plot_func: {e}")
            traceback.print_exc()
            print("callback:", callback)
            print("graph_type:", graph_type)
            print("kwargs:", kwargs)

    def plot(self, *args, name=None, labels=None, colors=None, graph_type=None, **kwargs):
        try:
            graph_type = graph_type or GraphType.Line
            kwargs = _merge_kwargs(kwargs, name=name, labels=labels, colors=colors)
            if (graph_type == GraphType.ParametricCurve) and (len(args) in (1, 2, 4)) and not callable(args[0]):
                return cls.parametric_curve(self, *args, **kwargs)
            if len(args) == 1:
                return plot_func(self, *args, graph_type=graph_type, **kwargs)
            if len(args) == 2:
                if graph_type == GraphType.Line:
                    return cls.line(self, *args, **kwargs)
            if len(args) == 3:
                return cls.colormap(self, *args, **kwargs)

            print(f"only 1, 2 or 3 arguments are supported, got {len(args)}")
            return None
        except Exception as e:
            print(f"Error in plot: {e}")
            traceback.print_exc()
            print("args:", args)
            print(f"name: {name}, labels: {labels}, colors: {colors}, graph_type: {graph_type}")
            print("kwargs:", kwargs)

    cls.plot = plot
    return cls



SciQLopPlot = _patch_sciqlop_plot(SciQLopPlot)
SciQLopTimeSeriesPlot = _patch_sciqlop_plot(SciQLopTimeSeriesPlot)
SciQLopMultiPlotPanel = _patch_sciqlop_plot(SciQLopMultiPlotPanel)
SciQLopNDProjectionPlot = _patch_sciqlop_plot(SciQLopNDProjectionPlot)




from SciQLopPlots import SciQLopPlot, SciQLopMultiPlotPanel
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtCore import Qt
import sys
import os
import math
import numpy as np
from types import SimpleNamespace

os.environ['QT_API'] = 'PySide6'

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow

def make_uniform_colormap(nx, ny):

    z = np.empty((nx,ny))
    x_vect= np.arange(nx)*8./nx -4.
    y_vect= np.arange(ny)*8./ny -4.

    z[:,:] = np.cos(np.sqrt((x_vect.reshape(-1,1)+2)**2 @ y_vect.reshape(-1,1).T**2))
    return x_vect, y_vect, z


class UniformColormap(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        self.plot(*make_uniform_colormap(200, 200))


class UniformColormap2DY(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        x, y, z = make_uniform_colormap(300, 200)
        y = np.array(np.repeat(y, 300).reshape(200, 300).T)
        self.plot(x, y, z)


class ColormapWithNonUniformXAxis(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self, parent, synchronize_x=False, synchronize_time=True)
        _x, y, _z = make_uniform_colormap(2010, 200)
        x = np.linspace(-4, 4, 210)
        z = np.empty((210, 200))
        x[:100] = _x[:1000:10]
        x[100:110] = _x[1000:1010]
        x[110:] = _x[1010::10]
        z[:100, :] = _z[:1000:10, :]
        z[100:110, :] = _z[1000:1010, :]
        z[110:, :] = _z[1010::10, :]
        self.plot(x, y, z)


if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(UniformColormap(w), "UniformColormap")
    w.add_tab(UniformColormap2DY(w), "UniformColormap2DY")
    w.add_tab(ColormapWithNonUniformXAxis(w), "ColormapWithNonUniformXAxis")
    w.show()
    app.exec()



from SciQLopPlots import (SciQLopPlot, MultiPlotsVerticalSpan,SciQLopMultiPlotPanel, SciQLopVerticalSpan,
                         SciQLopTimeSeriesPlot, GraphType, PlotType, AxisType, SciQLopPlotRange, PlotsModel,
                         InspectorView, SciQLopPixmapItem, SciQLopEllipseItem,
                         Coordinates)
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget, QTreeView
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush, QPixmap
from PySide6.QtCore import Qt, QTimer, QObject, QThread, Signal, QRectF
import sys, os
import math
import numpy as np
from datetime import datetime
from types import SimpleNamespace
import humanize

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

try:
    from numba import njit, prange
    NUMBA_AVAILABLE = True
except ImportError:
    NUMBA_AVAILABLE = False

os.environ['QT_API'] = 'PySide6'

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow


NPOINTS = 300000

def make_plot(parent, time_axis=False):
    if time_axis:
        plot = SciQLopTimeSeriesPlot(parent)
        plot.x_axis().set_range(datetime.now().timestamp(), datetime.now().timestamp()+NPOINTS)
    else:
        plot = SciQLopPlot(parent)
        plot.x_axis().set_range(SciQLopPlotRange(0, NPOINTS))
    plot.enable_legend(True)
    plot.minimize_margins()
    return plot

def add_graph(plot, time_axis=False, offset=0.):
    r = plot.x_axis().range()
    x=np.arange(start=r.start(), stop=r.stop(), dtype=np.float64)
    y=np.cos(np.array([x/6.,x/60.,x/600.])) * np.cos(np.array([x,x/6.,x/60.])) * [[100.],[1300],[17000]]
    y+=offset
    y=np.array(y.T, copy=True)
    return plot.plot(x,y, labels=["X","Y","Z"], colors=[QColorConstants.Red, QColorConstants.Blue, QColorConstants.Green])

def butterfly():
    t = np.linspace(0, 12*np.pi, 5000)
    x = np.sin(t)*(np.exp(np.cos(t)) - 2*np.cos(4*t) - np.sin(t/12)**5)
    y = np.cos(t)*(np.exp(np.cos(t)) - 2*np.cos(4*t) - np.sin(t/12)**5)
    return x, y

def add_curve(plot):
    x, y = butterfly()
    plot.x_axis().set_range(min(x), max(x))
    plot.y_axis().set_range(min(y), max(y))
    return plot.plot(x,y, labels=["butterfly"], colors=[QColorConstants.Red], graph_type=GraphType.ParametricCurve)

def add_colormap(plot, time_axis=False):
    x=np.arange(NPOINTS, dtype=np.float64)
    y=np.logspace(1, 4, 64)
    if time_axis:
        x+=datetime.now().timestamp()

    v_mod = np.tile(np.cos(np.arange(64)*6.28/(64*4)),NPOINTS).reshape(NPOINTS,64)
    noise = np.random.rand(NPOINTS*64).reshape(NPOINTS,64)
    h_mod = np.cos(np.arange(NPOINTS*64, dtype=np.float64)*6.28/(NPOINTS/3.1)).reshape(NPOINTS,64)
    sig = np.cos(np.arange(NPOINTS*64, dtype=np.float64)*6.28/(NPOINTS*10)).reshape(NPOINTS,64)
    z = (v_mod * sig * h_mod + noise)
    z = (z*sig)/(z + 1)
    return plot.plot(x,y,z, name = "Cmap", y_log_scale=True, z_log_scale=True)


class SimpleGraph(QWidget):
    def __init__(self,parent):
        QWidget.__init__(self,parent)
        self.setMouseTracking(True)
        self.plot = make_plot(self)
        self.setLayout(QVBoxLayout())
        self.layout().addWidget(self.plot)
        self.graph = add_graph(self.plot, time_axis=False, offset=2e4)

class SimpleCurve(QWidget):
    def __init__(self,parent):
        QWidget.__init__(self,parent)
        self.setMouseTracking(True)
        self.plot = make_plot(self)
        self.setLayout(QVBoxLayout())
        self.layout().addWidget(self.plot)
        self.pix = SciQLopPixmapItem(self.plot, QPixmap("://icons/SciQLop.png"), QRectF(-1, -1, 2, 2), True, Coordinates.Data)
        self.ellipse = SciQLopEllipseItem(self.plot, QRectF(-1, -1, 2, 2), QPen(Qt.red), QBrush(Qt.green), True, Coordinates.Data)
        self.curve = add_curve(self.plot)

class TimeSerieGraph(QWidget):
    def __init__(self,parent):
        QWidget.__init__(self,parent)
        self.setMouseTracking(True)
        self.plot = make_plot(self, time_axis=True)
        self.setLayout(QVBoxLayout())
        self.layout().addWidget(self.plot)

        self.graph = add_graph(self.plot, time_axis=True)

        x_range = self.plot.x_axis().range()

        self._verticalSpan = SciQLopVerticalSpan(self.plot, x_range/10, QColor(100, 100, 100, 100), read_only=False, visible=True, tool_tip="Vertical Span")

        self._ro_verticalSpan = SciQLopVerticalSpan(self.plot, x_range/20 + 100, QColor(200, 100, 100, 100), read_only=True, visible=True, tool_tip="Vertical Span")



class StackedPlots(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent)
        self.setMouseTracking(True)
        self.graphs = []
        for _ in range(5):
            plot = make_plot(None, time_axis=True)
            self.add_plot(plot)
            self.graphs.append(add_graph(plot, time_axis=True))

        self.cmap = add_colormap(self.plots()[-1], time_axis=True)

        x_range = self.plot_at(0).x_axis().range()

        self._verticalSpan = MultiPlotsVerticalSpan(self, x_range/10, QColor(100, 100, 100, 100), read_only=False, visible=True, tool_tip="Vertical Span")

if NUMBA_AVAILABLE:
    @njit(parallel=True)
    def _make_data(x,y):
        for i in prange(len(x)):
            y[i,0] = (math.cos(x[i]*(1./3.)) * math.cos(x[i]) +  np.random.rand() )* 100.
            y[i,1] = (math.cos(x[i]*(1./3.)) * math.cos(x[i]*(1./2.)) +  np.random.rand()) * 1300.
            y[i,2] = (math.cos(x[i]*(1./30.)) * math.cos(x[i]*(1./3.)) + math.cos(x[i]*(1./30.)) +  np.random.rand()) * 17000.
        return x, y

    def make_data( start, end):
        x = np.arange(start, end, dtype=np.float64)
        y = np.empty((len(x),3), dtype=np.float64)
        if len(x) > 100000:
            print(f"Using Numba to generate {humanize.metric(len(x))} points")
        return _make_data(x, y)


else:

    def make_data( start, end):
        x = np.arange(start, end, dtype=np.float64)
        y = np.empty((len(x), 3), dtype=np.float64)
        y[:,0] = np.cos(x*(1./3.)) * np.cos(x) * 100.
        y[:,1] = np.cos(x*(1./30.)) * np.cos(x*(1./3.)) * 1300.
        y[:,2] = np.cos(x*(1./300.)) * np.cos(x*(1./30.)) * 17000.
        return x, y



class DataProducers(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        for _ in range(3):
            self.plot(make_data,labels=["X","Y","Z"], colors=[QColorConstants.Red, QColorConstants.Blue, QColorConstants.Green], plot_type=PlotType.TimeSeries)
        plot = self.create_plot(-1, PlotType.TimeSeries)
        plot.plot(make_data,labels=["X","Y","Z"], colors=[QColorConstants.Red, QColorConstants.Blue, QColorConstants.Green], sync_with=plot.y_axis())


def fft(x, y, fft_size=2**12):
    if x is None or y is None:
        return np.array([]), np.array([])
    y = y[:, -1]
    freq = np.fft.rfftfreq(fft_size, x[1]-x[0])
    spec = np.zeros(len(freq), dtype='complex128')
    han = np.hanning(fft_size)
    nw = len(y)//fft_size
    try:
        if nw != 0:
            for i in range(nw):
                y_w = y[i*fft_size:(i+1)*fft_size]
                y_w = y_w-np.mean(y_w)
                y_w = y_w*han
                spec += (np.fft.rfft(y_w)/len(y_w))**2
            spec = np.abs(np.sqrt(spec/nw))
            return freq, spec
    except Exception as e:
        print(f"Error: {e}")

class TsAndFFT(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        _, self.graph = self.plot(make_data,labels=["X","Y","Z"], colors=[QColorConstants.Red, QColorConstants.Blue, QColorConstants.Green], plot_type=PlotType.TimeSeries)
        plot, _ = self.plot(fft,index=0,labels=["FFT"], colors=[QColorConstants.Red], sync_with=self.graph)
        plot.x_axis().set_log(True)
        plot.x_axis().set_range(0.01, 1)
        plot.y_axis().set_log(True)
        plot.y_axis().set_range(1., 1e-4)



class RealTimeAudio(SciQLopMultiPlotPanel):
    class AudioDataProducer(QThread):
        update_signal = Signal(object, object)
        def __init__(self,parent=None, size=2**10, sample_rate=48000):
            super().__init__(parent)
            self.moveToThread(self)
            try:
                import sounddevice as sd
                self._sd = sd
                sd.default.samplerate = sample_rate
                sd.default.channels = 1
                #sd.default.device = 'pipewire'
                self._size = size
                self.x = np.arange(self._size, dtype=np.float64)
                self.index = 0
                self.start()
            except ImportError:
                print("Sounddevice not found")

        def audio_callback(self, indata, frames, time, status):
            if status:
                print(status)
            if len(indata) == len(self.x):
                self.update_signal.emit(self.x, indata.astype(np.float64))


        def run(self):
            stream = self._sd.InputStream(callback=self.audio_callback, blocksize=self._size)
            with stream:
                while True:
                    self.sleep(1)

    class AudioSpectrogram:
        def __init__(self, size, fft_size, sample_rate, roll=True):
            self._size = size
            self._x = np.arange(size, dtype=np.float64)
            self._y = np.fft.rfftfreq(fft_size, 1./sample_rate)
            self._cutof = np.where(self._y>20000)[0][0]
            self._y = self._y[:self._cutof]
            self._z = np.zeros((size,len(self._y)), dtype=np.float64)
            self._last_x = len(self._z)-1
            self._roll = roll

        def __call__(self, x, y, *args):
            try:
                self._z[self._last_x] = y[:self._cutof]
                if self._roll:
                    self._z = np.roll(self._z, -1, axis=0)
                else:
                    self._last_x = (self._last_x + 1) % self._size
                return self._x, self._y, self._z
            except Exception as e:
                print(f"Error: {e}")
                return None

    def _update_data(self, x, y):
        self._graph.set_data(x, y)

    def __init__(self, parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        (self._plot,self._graph) = self.plot(np.arange(10)*1., np.arange(10)*1.,labels=["Audio"], colors=[QColorConstants.Blue])
        size = 2**12
        sample_rate = 192000
        plot, self._fft_graph = self.plot(lambda x,y:fft(x,y,size),index=0,labels=["FFT"], colors=[QColorConstants.Red], sync_with=self._graph)
        plot.x_axis().set_log(True)
        plot.x_axis().set_range(0.01, 1)
        plot.y_axis().set_log(True)
        plot.y_axis().set_range(1., 1e-4)
        self._audio_spec = RealTimeAudio.AudioSpectrogram(300,size,sample_rate)
        plot, _ = self.plot(self._audio_spec, name="Spectrogram",graph_type=GraphType.ColorMap, plot_type=PlotType.BasicXY, sync_with=self._fft_graph,y_log_scale=False,z_log_scale=True)
        self._data_producer = RealTimeAudio.AudioDataProducer(size=size, sample_rate=sample_rate)

        self._data_producer.update_signal.connect(self._update_data)



if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(SimpleGraph(w), "Simple Graph")
    w.add_tab(SimpleCurve(w), "Simple Curve")
    w.add_tab(TimeSerieGraph(w), "Time Serie Graph")
    w.add_tab(StackedPlots(w), "Stacked Plots")
    w.add_tab(DataProducers(w), "DataProducers")
    w.add_tab(TsAndFFT(w), "TsAndFFT")
    w.add_tab(RealTimeAudio(w), "RealTimeAudio")
    w.show()
    app.exec()



import SciQLopPlots
from SciQLopPlots import SciQLopPlot, MultiPlotsVerticalSpan,SciQLopMultiPlotPanel, SciQLopVerticalSpan, \
                         SciQLopTimeSeriesPlot, GraphType, PlotType, AxisType, SciQLopPlotRange, PlotsModel, InspectorView
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget, QTreeView
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
from PySide6.QtCore import Qt, QTimer, QObject, QThread, Signal
import sys, os
import math
import numpy as np
from datetime import datetime
from types import SimpleNamespace
import humanize

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

os.environ['QT_API'] = 'PySide6'
os.environ['QT_QPA_PLATFORM'] = 'xcb'

print("Using SciQLopPlots lib from:", SciQLopPlots.__file__)

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow



def fft(x, y, fft_size=2**12):
    if x is None or y is None:
        return np.array([]), np.array([])
    try:
        freq = np.fft.rfftfreq(fft_size, x[1]-x[0])
        spec = np.zeros(len(freq), dtype='complex128')
        han = np.hanning(fft_size)
        nw = len(y)//fft_size
        if nw != 0:
            for i in range(nw):
                y_w = y[i*fft_size:(i+1)*fft_size]
                y_w = y_w-np.mean(y_w)
                y_w = y_w*han
                spec += (np.fft.rfft(y_w)/len(y_w))**2
            spec = np.abs(np.sqrt(spec/nw))
            return freq, spec
    except Exception as e:
        print(f"Error: {e}")
        print(f"line: {sys.exc_info()[-1].tb_lineno}")


class RealTime(SciQLopMultiPlotPanel):
    class DataProducer(QThread):
        update_signal = Signal(object, object)
        def __init__(self,parent=None, size=2**10):
            super().__init__(parent)
            self.moveToThread(self)
            self._gen_size = size * 16
            gen_x = np.arange(self._gen_size, dtype=np.float64)
            self._size = size
            self._x = np.arange(size, dtype=np.float64)
            noise = np.random.rand(self._gen_size)*20.
            self._base = np.cos(gen_x/180.+noise)  * (np.cos(gen_x**2/18.**2+noise) * np.cos(gen_x/90.) * 100. + np.sin(gen_x/50.) * 50. +noise)
            self._index = 0
            self.start()

        def run(self):
            while True:
                self._index += 5
                self._index %= (len(self._base)-self._size)
                self.update_signal.emit(self._x, self._base[self._index:self._index+self._size])
                self.msleep(1)

    class Spectrogram:
        def __init__(self, size, fft_size, roll=True):
            sample_rate = 1.
            self._size = size
            self._x = np.arange(size, dtype=np.float64)
            self._freq = np.fft.rfftfreq(fft_size, sample_rate)
            self._z = np.zeros((size,len(self._freq)), dtype=np.float64)
            self._last_x = len(self._z)-1
            self._roll = roll

        def __call__(self, x, y, *args):
            try:
                self._z[self._last_x] = y
                if self._roll:
                    self._z = np.roll(self._z, -1, axis=0)
                else:
                    self._last_x = (self._last_x + 1) % self._size
                return self._x, self._freq, self._z
            except Exception as e:
                print(f"Error: {e}")
                print(f"line: {sys.exc_info()[-1].tb_lineno}")
                return None

    def _update_data(self, x, y):
        self._graph.set_data(x, y)

    def __init__(self, parent, with_spectro=True):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        (self._plot,self._graph) = self.plot(np.arange(10)*1., np.arange(10)*1.,labels=["Audio"], colors=[QColorConstants.Blue])
        size = 2**14
        plot, self._fft_graph = self.plot(lambda x,y:fft(x,y,size),index=0,labels=["FFT"], colors=[QColorConstants.Red], sync_with=self._graph)
        plot.x_axis().set_log(True)
        plot.x_axis().set_range(0.01, 1)
        plot.y_axis().set_log(True)
        plot.y_axis().set_range(1., 1e-4)
        if with_spectro:
            self._spec = RealTime.Spectrogram(500,size)
            plot, _ = self.plot(self._spec, name="Spectrogram",graph_type=GraphType.ColorMap, plot_type=PlotType.BasicXY, sync_with=self._fft_graph,y_log_scale=False,z_log_scale=True)
        self._data_producer = RealTime.DataProducer(size=size)

        self._data_producer.update_signal.connect(self._update_data, Qt.QueuedConnection)



if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.ipython_widget.close()
    w.add_tab(RealTime(w, with_spectro=False), "RealTime")
    w.showFullScreen()
    app.exec()



from PySide6 import *
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
import sys
import math
import numpy as np
from types import SimpleNamespace
from SciQLopPlots import SciQLopPlot,QCustomPlot, QCP, QCPColorMap, QCPRange, QCPColorScale, QCPAxis, QCPColorGradient, QCPMarginGroup


class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self._setup_ui()
        self.setGeometry(400, 250, 542, 390);


    def update_data(self, *args):
        x=np.arange(3e4)*10.
        y=np.ones((3,len(x)))
        y[0]=np.cos(x/60)
        y[1]=np.cos(x/600)*1.3
        y[2]=np.cos(x/6000)*1.7
        self.graph.setData(x,y)



    def _setup_ui(self):
        plot: SciQLopPlot = SciQLopPlot(self)
        plot.setInteractions(QCP.iRangeDrag | QCP.iRangeZoom | QCP.iSelectPlottables | QCP.iSelectAxes | QCP.iSelectLegend | QCP.iSelectItems)
        self.setCentralWidget(plot)
        self.graph = plot.addSciQLopGraph(plot.xAxis, plot.yAxis, ["X","Y","Z"])
        x=np.arange(3e7)*1.
        y=np.ones((3,len(x)))
        y[0]=np.cos(x/60)
        y[1]=np.cos(x/600)*1.3
        y[2]=np.cos(x/6000)*1.7
        self.graph.setData(x,y)
        self.graph.range_changed.connect(self.update_data)

        self.graph.graphAt(0).setPen(QPen(QColorConstants.Red))
        self.graph.graphAt(1).setPen(QPen(QColorConstants.Blue))
        self.graph.graphAt(2).setPen(QPen(QColorConstants.Green))

        plot.legend.setVisible(True)

        plot.rescaleAxes()
        #plot.setOpenGl(True)
        plot.enable_cursor(True)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    app.exec()



from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget
from PySide6.QtCore import Qt
import sys, os
import numpy as np
from datetime import datetime, timezone
import threading
from typing import Dict, Any
import platform

from SciQLopPlots import PropertiesPanel

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

os.environ['QT_API'] = 'PySide6'

if platform.system() == 'Linux':
    os.environ['QT_QPA_PLATFORM'] = os.environ.get("QT_QPA_PLATFORM", 'xcb')


def fix_name(name):
    return name.replace(" ", "_").replace(":", "_").replace("/", "_")

class Tabs(QTabWidget):
    def __init__(self,parent):
        QTabWidget.__init__(self,parent)
        self.setMouseTracking(True)
        self.setTabPosition(QTabWidget.TabPosition.North)
        self.setTabShape(QTabWidget.TabShape.Rounded)
        self.setMovable(True)

    def add_tab(self, widget, title):
        self.addTab(widget, title)
        widget.setObjectName(title)



class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setMouseTracking(True)
        self.axisRects=[]
        self.graphs = []
        self._setup_kernel()
        self._setup_ui()
        self.setGeometry(400, 250, 542, 390);
        self.setWindowTitle("SciQLopPlots Test")
        self.add_to_kernel_namespace('main_window', self)

    def add_tab(self, widget, title):
        self.tabs.add_tab(widget, title)
        self.add_to_kernel_namespace(fix_name(title), widget)

    def _setup_kernel(self):
        self.kernel_manager = QtInProcessKernelManager()
        self.kernel_manager.start_kernel(show_banner=False)
        self.kernel = self.kernel_manager.kernel
        self.kernel.gui = 'qt'

        self.kernel_client = self.kernel_manager.client()
        self.kernel_client.start_channels()

    def add_to_kernel_namespace(self, name, obj):
        self.kernel.shell.push({name: obj})

    def _setup_ui(self):
        self.tabs = Tabs(self)
        self.setCentralWidget(self.tabs)
        self.ipython_widget = RichJupyterWidget(self)
        dock = QDockWidget("IPython Console", self)
        dock.setWidget(self.ipython_widget)
        self.addDockWidget(Qt.BottomDockWidgetArea, dock)
        self.ipython_widget.kernel_manager = self.kernel_manager
        self.ipython_widget.kernel_client = self.kernel_client

        self.properties_panel = PropertiesPanel(self)
        dock = QDockWidget("Properties panel", self)
        dock.setWidget(self.properties_panel)
        self.addDockWidget(Qt.RightDockWidgetArea, dock)






import os
os.environ['QT_QPA_PLATFORM'] = "xcb"
from SciQLopPlots import SciQLopPlot, QCP, QCPColorMap, QCPItemRect, QCPRange, QCPColorScale, QCPAxis, QCPColorGradient, QCPMarginGroup
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush, Qt
import sys
import math
import numpy as np
from types import SimpleNamespace

from SciQLopPlots import SciQLopGraph , QCustomPlot, SciQLopVerticalSpan


if __name__ == '__main__':
    app = QApplication(sys.argv)
    p=SciQLopPlot()
    p.setOpenGl(True)

    p.setInteractions(QCP.iRangeDrag|QCP.iRangeZoom|QCP.iSelectPlottables|QCP.iSelectItems)
    graph = p.addSciQLopGraph(p.xAxis, p.yAxis, ["X","Y","Z"])
    x=np.arange(3e7)*1.
    y=np.ones((3,len(x)))
    y[0]=np.cos(x/60)
    y[1]=np.cos(x/600)*1.3
    y[2]=np.cos(x/6000)*1.7
    graph.setData(x,y)

    graph.graphAt(0).setPen(QPen(QColorConstants.Red))
    graph.graphAt(1).setPen(QPen(QColorConstants.Blue))
    graph.graphAt(2).setPen(QPen(QColorConstants.Green))

    p.legend.setVisible(True)

    p.rescaleAxes()


    spans = [SciQLopVerticalSpan(p, QCPRange(i*20., i*20.+10.)) for i in range(5)]
    for i in range(5):
        spans[i].setToolTip(f"Vertical span {i}")


    p.show()
    app.exec()



from SciQLopPlots import SciQLopPlot, QCP, QCPColorMap, QCPRange, QCPColorScale, QCPAxis, \
                         QCPLegend, QCPColorGradient, QCPMarginGroup, QCPAxisRect
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
from PySide6.QtCore import Qt
import sys
import math
import numpy as np
from types import SimpleNamespace


class PlotPanel(QScrollArea):
    def __init__(self,parent):
        QScrollArea.__init__(self,parent)
        self.setWidget(QWidget(self))
        self.widget().setLayout(QVBoxLayout())
        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.setWidgetResizable(True)

    def set_plot(self, plot):
        self.widget().layout().addWidget(plot)



class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.axisRects=[]
        self.graphs = []
        self._setup_ui()
        self.setGeometry(400, 250, 542, 390);

    def _setup_ui(self):
        plot: SciQLopPlot = SciQLopPlot(self)
        plot.setOpenGl(True)
        plot.setInteractions(QCP.iRangeDrag|QCP.iRangeZoom|QCP.iSelectPlottables)
        plot.plotLayout().clear()
        self.plot_panel = PlotPanel(self)
        self.setCentralWidget(self.plot_panel)
        self.plot_panel.set_plot(plot)
        for i in range(10):
            axis_rect = QCPAxisRect(plot)
            plot.legend = QCPLegend()
            axis_rect.insetLayout().addElement(plot.legend, Qt.AlignBottom|Qt.AlignRight)
            plot.legend.setVisible(True)
            axis_rect.setMinimumSize(200,200)
            self.axisRects.append(axis_rect)
            plot.plotLayout().addElement(len(self.axisRects)-1,0,axis_rect)
            graph = plot.addSciQLopGraph(axis_rect.axis(QCPAxis.atBottom), axis_rect.axis(QCPAxis.atLeft), ["X","Y","Z"])
            self.graphs.append(graph)
            x=np.arange(300000)*1.
            y=np.ones((3,len(x)))
            y[0]=np.cos(x/60)
            y[1]=np.cos(x/600)*1.3
            y[2]=np.cos(x/6000)*1.7
            graph.setData(x,y)

            graph.graphAt(0).setPen(QPen(QColorConstants.Red))
            graph.graphAt(1).setPen(QPen(QColorConstants.Blue))
            graph.graphAt(2).setPen(QPen(QColorConstants.Green))

        #plot.legend.setVisible(True)

        plot.rescaleAxes()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    app.exec()



import SciQLopPlots
from SciQLopPlots import SciQLopPlot, MultiPlotsVerticalSpan,SciQLopMultiPlotPanel, SciQLopVerticalSpan, \
                         SciQLopTimeSeriesPlot, GraphType, PlotType, AxisType, SciQLopPlotRange, PlotsModel, InspectorView
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget, QTreeView
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
from PySide6.QtCore import Qt, QTimer, QObject, QThread, Signal
import sys, os
import math
import numpy as np
from datetime import datetime, timezone
from types import SimpleNamespace
import humanize

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager
from numba import jit, prange

os.environ['QT_API'] = 'PySide6'
os.environ['QT_QPA_PLATFORM'] = 'xcb'

print(f"Using SciQLopPlots lib from: {SciQLopPlots.__file__}, {SciQLopPlots.SciQLopPlotsBindings.__file__}")


sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow

@jit(nopython=True, parallel=True)
def compute_data(size):
    """Compute a large array of random data."""
    data = np.zeros(size, dtype=np.float64)
    for i in prange(size):
        data[i] = (np.cos(i / 180.) * (np.cos(i**2 / 18.**2) * np.cos(i / 90.) * 100. +
                                        np.sin(i / 50.) * 50.) + np.random.rand() * 10.)
    return data

def generate_data(size=int(1e9)):
    """Generate a large array of random data."""
    # Simulate a large dataset
    time_vector = np.arange(size, dtype=np.float64)
    return time_vector, compute_data(size)

class GigaPoints(SciQLopMultiPlotPanel):
    def __init__(self, parent, size=1e9):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        time, data = generate_data(int(size))  # Generate 1 billion points
        (self._plot, self._graph) = self.plot(time, data, labels=[f"{humanize.scientific(size)} points"], colors=[QColorConstants.Blue])



if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.ipython_widget.close()
    w.add_tab(GigaPoints(w, size=1e9), "GigaPoints")
    w.showFullScreen()
    app.exec()




"""
The idea of this test is to create an delete continously plot panels until it
 crashes in order to catch race conditions or wrong destruction order.
"""

from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget
from PySide6.QtGui import QColorConstants, QColor
from PySide6.QtCore import Qt, QTimer
import sys, os
import numpy as np
from datetime import datetime, timezone
import threading
from typing import Dict, Any
from SciQLopPlots import (SciQLopPlot, MultiPlotsVerticalSpan,SciQLopMultiPlotPanel, SciQLopVerticalSpan,
                         SciQLopTimeSeriesPlot, GraphType, PlotType, AxisType, SciQLopPlotRange, PlotsModel,
                         InspectorView, SciQLopPixmapItem, SciQLopEllipseItem,
                         Coordinates)
import random


sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow


os.environ['QT_API'] = 'PySide6'

NPOINTS = 30000


def make_plot(parent, time_axis=False):
    if time_axis:
        plot = SciQLopTimeSeriesPlot(parent)
        plot.x_axis().set_range(datetime.now().timestamp(), datetime.now().timestamp()+NPOINTS)
    else:
        plot = SciQLopPlot(parent)
        plot.x_axis().set_range(SciQLopPlotRange(0, NPOINTS))
    plot.enable_legend(True)
    plot.minimize_margins()
    return plot


def add_graph(plot, time_axis=False, offset=0.):
    r = plot.x_axis().range()
    x=np.arange(start=r.start(), stop=r.stop(), dtype=np.float64)
    y=np.cos(np.array([x/6.,x/60.,x/600.])) * np.cos(np.array([x,x/6.,x/60.])) * [[100.],[1300],[17000]]
    y+=offset
    y=np.array(y.T, copy=True)
    return plot.plot(x,y, labels=["X","Y","Z"], colors=[QColorConstants.Red, QColorConstants.Blue, QColorConstants.Green])


class StackedPlots(SciQLopMultiPlotPanel):
    def __init__(self, parent):
        SciQLopMultiPlotPanel.__init__(self, parent)
        self.setMouseTracking(True)
        self.graphs = []
        for _ in range(3):
            plot = make_plot(None, time_axis=True)
            self.add_plot(plot)
            self.graphs.append(add_graph(plot, time_axis=True))

        x_range = self.plot_at(0).x_axis().range()
        self._sub_panel = SciQLopMultiPlotPanel(self,
                                               synchronize_x=False,
                                               synchronize_time=True,
                                               orientation=Qt.Orientation.Horizontal)
        self.add_panel(self._sub_panel)
        for _ in range(2):
            plot = make_plot(None, time_axis=True)
            self._sub_panel.add_plot(plot)
            self.graphs.append(add_graph(plot, time_axis=True))
        self._verticalSpan = MultiPlotsVerticalSpan(self, x_range/10, QColor(100, 100, 100, 100), read_only=False, visible=True, tool_tip="Vertical Span")
        self.setAttribute(Qt.WA_DeleteOnClose)


def create_plot_panel(main_window):
    main_window.tabs.add_tab(StackedPlots(main_window), f"StackedPlots-{main_window.tabs.count()}")


def remove_plot_panel(main_window):
    if main_window.tabs.count() == 0:
        return
    random_tab_index = np.random.randint(0, main_window.tabs.count())
    main_window.tabs.widget(random_tab_index).close()


def random_add_or_remove_panels(main_window):
    random.choices([create_plot_panel, remove_plot_panel],
                   weights=[10, main_window.tabs.count()])[0](main_window)


if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w._timer = QTimer(w)
    w._timer.timeout.connect(lambda: random_add_or_remove_panels(w))
    w._timer.setInterval(100)
    w._timer.start()
    create_plot_panel(w)
    w.add_to_kernel_namespace("create_plot_panel", lambda: create_plot_panel(w))
    w.add_to_kernel_namespace("remove_plot_panel", lambda index: w.tabs.widget(index).close())
    w.add_to_kernel_namespace("stop_timer", w._timer.stop)
    w.add_to_kernel_namespace("start_timer",  w._timer.start)
    w.show()
    app.exec()



from SciQLopPlots import (SciQLopPlot,
                         MultiPlotsVerticalSpan ,SciQLopMultiPlotPanel,
                         SciQLopVerticalSpan, SciQLopTimeSeriesPlot,
                         PlotType, AxisType, GraphType, SciQLopVerticalLine,
                         SciQLopHorizontalLine)
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget
from PySide6.QtGui import QColorConstants
from PySide6.QtCore import Qt
import sys, os
import numpy as np
from datetime import datetime, timezone
import threading
from typing import Dict, Any

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow


os.environ['QT_API'] = 'PySide6'

try:
    import speasy as spz
    from speasy.signal.resampling import resample
    from speasy.core import datetime64_to_epoch

    def _current_thread_id():
        return threading.get_native_id()


    class ThreadStorage:
        _storage: Dict[int, Dict[str, Any]] = {}

        def __init__(self):
            self._storage = {}

        def __getattr__(self, item):
            return self._storage.get(_current_thread_id(), {}).get(item, None)

        def __setattr__(self, key, value):
            tid = _current_thread_id()
            storage = self._storage
            if tid not in storage:
                storage[tid] = {}
            storage[tid][key] = value

    from speasy.core.cache import _cache
    _cache._data._local = ThreadStorage()

except ImportError:
    print("Speasy not found")
    exit()


def mms1_fgm_b_gse_srvy_l2(start, stop):
    try:
        v = spz.get_data(
                        spz.inventories.tree.cda.MMS.MMS1.FGM.MMS1_FGM_SRVY_L2.mms1_fgm_b_gse_srvy_l2,
                        start, stop)
        if v is None:
            return None
        v = resample(v, 1./4)
        x = datetime64_to_epoch(v.time)
        return x, v.values.astype(np.float64)
    except Exception as e:
        print(f"Error: {e}")
        return None

def speasy_spectro(product, start, stop, invert_y=False):
    try:
        v=spz.get_data(product, start, stop)
        if v is None:
            return None
        x=datetime64_to_epoch(v.time)
        if invert_y:
            return x, v.axes[1].values.astype(np.float64)[::-1].copy() ,v.values.astype(np.float64)[:,::-1].copy()
        return x, v.axes[1].values.astype(np.float64) ,v.values.astype(np.float64)
    except Exception as e:
        print(f"Error: {e}")
        return None

def mms1_edp_hmfe_dsl_brst_l2(start, stop):
    try:
        v=spz.get_data(spz.inventories.tree.cda.MMS.MMS1.ADP_SDP.MMS1_EDP_BRST_L2_HMFE.mms1_edp_hmfe_dsl_brst_l2, start, stop)
        if v is None:
            return None
        x=datetime64_to_epoch(v.time)
        return x, v.values.astype(np.float64)
    except Exception as e:
        print(f"Error: {e}")
        return None

class Spectrum:
    def __init__(self, fft_size):
        self._fft_size = fft_size

    def __call__(self, x, y):
        try:
            if x is None or y is None:
                return np.array([]), np.array([])
            y = y[:, -1]
            spec = np.zeros(self._fft_size, dtype='complex128')
            han = np.hanning(self._fft_size)
            nw = len(y)//self._fft_size
            if nw != 0:
                for i in range(nw):
                    y_w = y[i*self._fft_size:(i+1)*self._fft_size]
                    y_w = y_w-np.mean(y_w)
                    y_w = y_w*han
                    spec += (np.fft.fft(y_w)/len(y_w))**2
                spec = np.abs(np.sqrt(spec/nw))
                freq = np.fft.fftfreq(len(spec), x[1]-x[0])[1:len(spec)//2]
                spec = spec[1:len(spec)//2]
                return freq, spec
        except Exception as e:
            print(f"Error: {e}")
            return None

class MMS(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        self.plot(lambda start, stop: speasy_spectro('cda/MMS1_FPI_FAST_L2_DIS-MOMS/mms1_dis_energyspectr_omni_fast', start, stop),
                  name="mms1_dis_energyspectr_omni_fast",
                  graph_type=GraphType.ColorMap,
                  plot_type=PlotType.TimeSeries,
                  y_log_scale=True,
                  z_log_scale=True)
        _, self.graph = self.plot(mms1_fgm_b_gse_srvy_l2,
                             labels=['Bx GSE', 'By GSE', 'Bz GSE', 'Bt'],
                             colors=[QColorConstants.Red, QColorConstants.Green, QColorConstants.Blue, QColorConstants.Black],
                             plot_type=PlotType.TimeSeries)
        self._fft = Spectrum(2**9)
        p,g=self.plot(self._fft, index=1, labels=['Spectrum'], colors=[QColorConstants.Red], plot_type=PlotType.BasicXY, sync_with=self.graph)
        p.x_axis().set_log(True)
        p.x_axis().set_range(0.1, 2)
        p.y_axis().set_log(True)
        p.y_axis().set_range(1., 1e-4)
        self._g = g
        self.set_time_axis_range(datetime(2019,2,17,12,33,0,0,timezone.utc), datetime(2019,2,17,12,34,0,0,timezone.utc))

class MMS_Spectro_Only(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        self.plot(lambda start, stop: speasy_spectro('cda/MMS1_FPI_FAST_L2_DIS-MOMS/mms1_dis_energyspectr_omni_fast', start, stop),
                  name="mms1_dis_energyspectr_omni_fast",
                  graph_type=GraphType.ColorMap,
                  plot_type=PlotType.TimeSeries,
                  y_log_scale=True,
                  z_log_scale=True)
        self.plot(lambda start, stop: speasy_spectro('cda/MMS1_FPI_BRST_L2_DIS-MOMS/mms1_dis_energyspectr_omni_brst', start, stop),
                  name="mms1_dis_energyspectr_omni_brst",
                  graph_type=GraphType.ColorMap,
                  plot_type=PlotType.TimeSeries,
                  y_log_scale=True,
                  z_log_scale=True)
        self.set_time_axis_range(datetime(2019,2,17,12,33,0,0,timezone.utc), datetime(2019,2,17,12,34,0,0,timezone.utc))

class MMS_edp_hmfe(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        _, graph = self.plot(mms1_edp_hmfe_dsl_brst_l2,
                             labels=['Ex', 'Ey', 'Ez'],
                             colors=[QColorConstants.Red, QColorConstants.Green, QColorConstants.Blue],
                             plot_type=PlotType.TimeSeries)

        self.set_time_axis_range(datetime(2023,10,20,7,0,0,0,timezone.utc), datetime(2023,10,20,9,0,0,0,timezone.utc))

class Maven_swia_spectra_diff_en_fluxes(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        self.plot(lambda start, stop: speasy_spectro('cda/MVN_SWI_L2_ONBOARDSVYSPEC/spectra_diff_en_fluxes', start, stop, True),
                  name="spectra_diff_en_fluxes",
                  graph_type=GraphType.ColorMap,
                  plot_type=PlotType.TimeSeries,
                  y_log_scale=True,
                  z_log_scale=True)

        self.set_time_axis_range(datetime(2022,2,13,0,0,0,0,timezone.utc), datetime(2022,2,14,0,0,0,0,timezone.utc))

if __name__ == '__main__':
    from speasy.core.cache import _cache
    _cache._data._local = ThreadStorage()
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(MMS(w), "MMS")
    w.add_tab(MMS_Spectro_Only(w), "MMS Spectro Only")
    w.add_tab(MMS_edp_hmfe(w), "MMS EDP HMFE")
    w.add_tab(Maven_swia_spectra_diff_en_fluxes(w), "Maven SWIA Spectra Diff En Fluxes")
    w.show()
    app.exec()



from PySide6.QtCore import Qt
from SciQLopPlots import SciQLopPlot
from PySide6.QtGui import QPainter, QPen, QImage, QIcon
from PySide6.QtWidgets import QApplication, QWidget, QStyledItemDelegate, QComboBox, QVBoxLayout
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow


os.environ['QT_API'] = 'PySide6'


class DrawingAPITester(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setWindowTitle("Drawing API Tester")
        self.resize(400, 400)

    def paintEvent(self, event):
        painter = QPainter(self)
        #painter.setRenderHint(QPainter.Antialiasing)

        pen = QPen(Qt.black)
        pen.setWidth(2)
        painter.setPen(pen)
        painter.drawRect(100, 100, 200, 200)
        painter.translate(100, 100)
        painter.drawImage(0, 0, QImage(":/LineStyles/StepCenter.png").scaledToWidth(100*painter.device().devicePixelRatio()))
        painter.translate(100, 100)
        painter.drawImage(0, 0, QImage(":/LineStyles/StepCenter.png").scaledToWidth(100))


class CustomDelegate(QStyledItemDelegate):
    def __init__(self, parent):
        super().__init__(parent)

    def paint(self, painter, option, index):
        painter.save()
        painter.translate(option.rect.topLeft())
        text = index.data()
        print(f":/LineStyles/{text}.png")
        painter.drawImage(0, 0, QImage(f":/LineStyles/{text}.png").scaledToHeight(option.rect.height()/painter.device().devicePixelRatio()))
        painter.restore()


class TestCombox(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._layout = QVBoxLayout()
        self.setLayout(self._layout)
        self._combo = QComboBox()
        self._layout.addStretch()
        self._layout.addWidget(self._combo)
        self._layout.addStretch()
        for i in ("Line", "NoLine", "StepCenter", "StepLeft", "StepRight"):
            self._combo.addItem(i)
            #self._combo.addItem(QIcon(f":/LineStyles/{i}.png"), i)
        self._combo.setItemDelegate(CustomDelegate(self))



if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(DrawingAPITester(), "Drawing API Tester")
    w.add_tab(TestCombox(), "Test Combox")
    w.show()
    app.exec()



from SciQLopPlots import SciQLopPlot, \
                         MultiPlotsVerticalSpan ,SciQLopMultiPlotPanel, SciQLopVerticalSpan, \
                         SciQLopTimeSeriesPlot, PlotType, AxisType, GraphType, SciQLopNDProjectionPlot
from PySide6.QtWidgets import QMainWindow, QApplication, QScrollArea,QWidget, QVBoxLayout, QTabWidget, QDockWidget
from PySide6.QtGui import QColorConstants
from PySide6.QtCore import Qt
import sys, os
import numpy as np
from datetime import datetime, timezone
import threading
from typing import Dict, Any

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow

os.environ['QT_API'] = 'PySide6'

try:
    import speasy as spz
    from speasy.signal.resampling import resample
    def _current_thread_id():
        return threading.get_native_id()


    class ThreadStorage:
        _storage: Dict[int, Dict[str, Any]] = {}

        def __init__(self):
            self._storage = {}

        def __getattr__(self, item):
            return self._storage.get(_current_thread_id(), {}).get(item, None)

        def __setattr__(self, key, value):
            tid = _current_thread_id()
            storage = self._storage
            if tid not in storage:
                storage[tid] = {}
            storage[tid][key] = value

    from speasy.core.cache import _cache
    _cache._data._local = ThreadStorage()

except ImportError:
    print("Speasy not found")
    exit()


def spc_pos_gse(spacecraft, start, stop):
    try:
        v=spz.get_data(f"amda/{spacecraft}_xyz_gse", start, stop)
        if v is None:
            return None
        t=(v.time.astype(np.int64)/1e9).astype(np.float64)
        x = v.values[:,0].astype(np.float64)
        y = v.values[:,1].astype(np.float64)
        z = v.values[:,2].astype(np.float64)
        return t, x, y, z
    except Exception as e:
        print(f"Error: {e}")
        return None

class MMS(SciQLopMultiPlotPanel):
    def __init__(self,parent):
        SciQLopMultiPlotPanel.__init__(self,parent, synchronize_x=False, synchronize_time=True)
        proj,_ = self.plot(lambda start,stop: spc_pos_gse("mms1", start, stop), labels=['mms1']*3, graph_type=GraphType.ParametricCurve,
                            plot_type=PlotType.Projections)
        for i in range(2,4):
            proj.plot(lambda start,stop: spc_pos_gse(f"mms{i+1}", start, stop), labels=[f'mms{i+1}']*3, graph_type=GraphType.ParametricCurve)
        for i in range(4):
            proj.plot(lambda start,stop: spc_pos_gse(f"c{i+1}", start, stop), labels=[f'c{i+1}']*3, graph_type=GraphType.ParametricCurve)
        self.set_time_axis_range(datetime(2019,2,17,12,33,0,0,timezone.utc), datetime(2019,2,17,12,34,0,0,timezone.utc))



if __name__ == '__main__':
    from speasy.core.cache import _cache
    _cache._data._local = ThreadStorage()
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(MMS(w), "MMS")
    w.show()
    app.exec()



from SciQLopPlots import SciQLopPlot, \
                         MultiPlotsVerticalSpan ,SciQLopMultiPlotPanel, SciQLopVerticalSpan, \
                         SciQLopTimeSeriesPlot, PlotType, AxisType, GraphType, PlotDragNDropCallback
from PySide6.QtWidgets import QMainWindow, QApplication, QListWidget ,QWidget, QSplitter, QHBoxLayout, QTabWidget, QDockWidget
from PySide6.QtGui import QColorConstants
from PySide6.QtCore import Qt, QMimeData
import sys, os
import numpy as np
from datetime import datetime, timezone
import threading
from typing import Dict, Any

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
from common import MainWindow


os.environ['QT_API'] = 'PySide6'

class DNDSource(QListWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.addItems(["Cos", "Sin", "Rand"])
        self.setDragEnabled(True)

    def mimeData(self, items):
        mime_data = QMimeData()
        mime_data.setData("text/plain", (";".join([item.text() for item in items])).encode())
        return mime_data


class MyCallback(PlotDragNDropCallback):
    def __init__(self,parent, source):
        super().__init__("text/plain", True, parent)
        self.source = source

    def call(self, plot, mime_data:QMimeData):
        print(f"Called with plot {plot}, {mime_data}")
        data = mime_data.data("text/plain").data().decode()
        print(data)
        x = np.linspace(0, 2*np.pi, 100)
        if data == "Cos":
            plot.plot(x, np.cos(x), labels=["cos"], colors=[QColorConstants.DarkGreen])
        elif data == "Sin":
            plot.plot(x, np.sin(x), labels=["sin"], colors=[QColorConstants.DarkBlue])
        elif data == "Rand":
            plot.plot(x, np.random.rand(len(x)), labels=["rand"], colors=[QColorConstants.DarkRed])



class DNDTarget(SciQLopMultiPlotPanel):
    def __init__(self, parent=None, source=None):
        super().__init__(parent,synchronize_x=False, synchronize_time=True)
        self._callback = MyCallback(self, source=source)
        self.add_accepted_mime_type(self._callback)


class DNDTest(QSplitter):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)
        self.source = DNDSource(self)
        self.addWidget(self.source)
        self.target = DNDTarget(self, source=self.source)
        self.addWidget(self.target)


if __name__ == '__main__':
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL, True)
    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
    app = QApplication(sys.argv)
    w = MainWindow()
    w.add_tab(DNDTest(w), "DND Test")
    w.show()
    app.exec()



#!/bin/env python3

import os
import sys
import importlib
import argparse
from glob import glob
import subprocess
import json
import platform

parser = argparse.ArgumentParser(description='PySide/shiboken generator wrapper')
parser.add_argument('--input-header')
parser.add_argument('--input-xml')
parser.add_argument('--shiboken')
parser.add_argument('--typesystem-paths')
parser.add_argument('--output-directory')
parser.add_argument('--build-directory')
parser.add_argument('--ref-build-target')
args = parser.parse_args()

env = os.environ.copy()

def cpp_flags(build_dir, ref_build_target):
    with open(f"{build_dir}/meson-info/intro-targets.json", 'r') as targets_f:
        targets = json.load(targets_f)
        # this is extra fragile!
        ref_target = list(filter(lambda k: k['name'].startswith(ref_build_target), targets))[0]
        params = ref_target['target_sources'][0]['parameters']
        return list(filter(lambda p: p.upper().startswith('-I') or p.startswith('-F'), params))


shiboken_constant_args=['--generator-set=shiboken',
    '--enable-parent-ctor-heuristic',
    '--enable-return-value-heuristic',
    '--use-isnull-as-nb_nonzero',
    '--avoid-protected-hack',
    '--enable-pyside-extensions',
    #'--debug-level=full',
    '-std=c++20',
    '--generator-set=shiboken']

if 'linux' in platform.system().lower():
    #if os.environ.get('AUDITWHEEL_PLAT', '') == 'manylinux_2_28_x86_64':
    #    gcc_found = False
    #    for v in reversed(range(11,14)):
    #        prefix = f'/opt/rh/gcc-toolset-{v}/root/usr'
    #        if os.path.exists(prefix):
    #            shiboken_constant_args += [
    #                f"-I/opt/rh/gcc-toolset-{v}/root/usr/lib/gcc/x86_64-redhat-linux/{v}/../../../../include/c++/{v}",
    #                f"-I/opt/rh/gcc-toolset-{v}/root/usr/lib/gcc/x86_64-redhat-linux/{v}/../../../../include/c++/{v}/x86_64-redhat-linux",
    #                f"-I/opt/rh/gcc-toolset-{v}/root/usr/lib/gcc/x86_64-redhat-linux/{v}/../../../../include/c++/{v}/backward",
    #                f"-I/opt/rh/gcc-toolset-{v}/root/usr/lib/gcc/x86_64-redhat-linux/{v}/include",
    #                f"-I/opt/rh/gcc-toolset-13/root/usr/include",
    #                f"-I/usr/local/include",
    #                f"-I/usr/include",
    #            ]
    #            gcc_found = True
    #            shiboken_constant_args += ['--compiler=g++']
    #            break
    try:
        llvm_dir = subprocess.run(['llvm-config', '--prefix'], check=True, stdout=subprocess.PIPE)
        if llvm_dir.returncode == 0:
            llvm_dir = llvm_dir.stdout.decode().strip()
            env['LLVM_INSTALL_DIR'] = llvm_dir
    except FileNotFoundError:
        pass

cmd = [args.shiboken, args.input_header, args.input_xml ] + shiboken_constant_args + cpp_flags(args.build_directory, args.ref_build_target) + [ f'--typesystem-paths={args.typesystem_paths}', f'--output-directory={args.output_directory}']

try:
    r = subprocess.run(
        cmd,
        check=True,
        env=env,
        capture_output=True
    )
except subprocess.CalledProcessError as e:
    print(e)
    print(e.stdout.decode())
    print(e.stderr.decode())
    sys.exit(e.returncode)

print(r.stdout.decode())
print(r.stderr.decode())
sys.exit(r.returncode)
